@@ -17,9 +17,8 @@ def train(
         *,
         fit_f, eval_f=None, net_init=None, get_net=None, get_loss=None, get_trainer=None, save_params=None,
         enable_hyper_search=False, reporthook=None, final_reporthook=None, primary_key=None,
-        eval_epoch=1, loss_dict2tmt_loss=None,
+        eval_epoch=1, loss_dict2tmt_loss=None, epoch_lr_scheduler=None, batch_lr_scheduler=None,
         **cfg_kwargs):
-
     net = net if get_net is None else get_net(**cfg.hyper_params)
 
     if net_init is not None:
@@ -64,7 +63,8 @@ def train(
             total_epoch=cfg.end_epoch - 1
         )
     elif progress_monitor is None or progress_monitor == "tqdm":
-        progress_monitor = lambda x, e: tqdm(x, "Epoch: %s" % e)
+        def progress_monitor(x, e):
+            return tqdm(x, "Epoch: %s" % e)
 
     if dump_result:
         from longling import config_logging
@@ -82,12 +82,17 @@ def train(
         evaluation_formatter = Formatter()
 
     # train check
-    trainer = get_trainer(
-        net, optimizer=cfg.optimizer,
-        optimizer_params=cfg.optimizer_params,
-        select=cfg.train_select,
-        lr_params=cfg.lr_params
-    ) if get_trainer is not None else trainer
+    if get_trainer is not None:
+        trainer = get_trainer(
+            net, optimizer=cfg.optimizer,
+            optimizer_params=cfg.optimizer_params,
+            select=cfg.train_select,
+            lr_params=cfg.lr_params
+        )
+        if batch_lr_scheduler is True:
+            trainer, batch_lr_scheduler = trainer
+        elif epoch_lr_scheduler is True:
+            trainer, epoch_lr_scheduler = trainer
 
     for epoch in range(cfg.begin_epoch, cfg.end_epoch):
         for i, batch_data in enumerate(progress_monitor(train_data, epoch)):
@@ -98,6 +103,8 @@ def train(
                 loss_monitor=loss_monitor,
                 ctx=ctx,
             )
+            if batch_lr_scheduler is not None:
+                batch_lr_scheduler.step()
 
         if cfg.lr_params and "update_params" in cfg.lr_params and cfg.end_epoch - cfg.begin_epoch - 1 > 0:
             cfg.logger.info("reset trainer")
@@ -143,5 +150,8 @@ def train(
             cfg.logger.info("save model params to %s, with select='%s'" % (params_path, cfg.save_select))
             save_params(params_path, net, select=cfg.save_select)
 
+        if epoch_lr_scheduler is not None:
+            epoch_lr_scheduler.step()
+
     if final_reporthook is not None:
         final_reporthook()
