@@ -6,7 +6,7 @@ import warnings
 from heapq import nlargest
 from longling.ML.toolkit.analyser import get_max, get_min, get_by_key, key_parser
 from longling import Configuration, path_append
-from longling import dict2pv, list2dict
+from longling import list2dict, nested_update
 
 import json
 import sqlite3
@@ -29,10 +29,12 @@ def _key(x):
 
 
 def show(key, max_key=True, exp_id=None, res_dir="./",
-         nni_dir=path_append(os.environ.get("HOME", "./"), "nni/experiments"),
+         nni_dir=path_append(os.environ.get("HOME", "./"), "nni-experiments"),
          only_final=False,
          with_keys=None, with_all=False):  # pragma: no cover
     """
+    Updated in v1.3.17
+
     cli alias: ``nni show``
 
     Parameters
@@ -84,8 +86,10 @@ def show(key, max_key=True, exp_id=None, res_dir="./",
 
 
 def show_top_k(k, exp_id=None,
-               exp_dir=path_append(os.environ.get("HOME", "./"), "nni/experiments")):  # pragma: no cover
+               exp_dir=path_append(os.environ.get("HOME", "./"), "nni-experiments")):  # pragma: no cover
     """
+    Updated in v1.3.17
+
     cli alias: ``nni k-best``
 
     Parameters
@@ -98,6 +102,9 @@ def show_top_k(k, exp_id=None,
     -------
 
     """
+    import warnings
+    warnings.warn("deprecated method")
+
     if exp_id:
         exp_dir = path_append(exp_dir, exp_id)
     sqlite_db = path_append(exp_dir, "db", "nni.sqlite", to_str=True)
@@ -125,55 +132,51 @@ class BaseReporter(object):
         raise NotImplementedError
 
 
-def get_params(received_params: dict, cfg_cls: (Configuration, type(Configuration))):
+def get_params(received_params: dict):
     """
+    Updated in v1.3.17
+
     Parameters
     ----------
     received_params: dict
         nni get_next_parameters() 得到的参数字典
-    cfg_cls: Configuration
-        配置文件实例
 
     Returns
     -------
     cfg_params: dict
-        存在与 cfg 中 received_params 的更新值
-    unk_params: dict
-        不存在与 cfg 中 received_params 的更新值
+        更新后的参数字典
 
     Examples
     --------
-    >>> class CFG(Configuration):
-    ...     hyper_params = {"hidden_num": 100}
-    ...     learning_rate = 0.001
-    >>> cfg = CFG()
-    >>> get_params({"hidden_num": 50, "learning_rate": 0.1, "act": "relu"}, cfg)
-    ({'hyper_params': {'hidden_num': 50}, 'learning_rate': 0.1}, {'act': 'relu'})
+    >>> get_params({
+    ...     "hyper_params_update:hidden_num": 50,
+    ...     "hyper_params_update:alpha": 5,
+    ...     "learning_rate": 0.1
+    ... })
+    {'hyper_params_update': {'hidden_num': 50, 'alpha': 5}, 'learning_rate': 0.1}
     """
     cfg_params = {}
-    unk_params = {}
-
-    path, _ = dict2pv(cfg_cls.vars())
-
-    keys = {p[-1]: p for p in path}
 
     for k, v in received_params.items():
-        if k in cfg_cls.vars():
-            cfg_params[k] = v
+        if ":" in k:
+            k = k.split(":")
+            obj = list2dict(k, v)
         else:
-            if k in keys:
-                cfg_params.update(list2dict(keys[k], v))
-            else:
-                unk_params[k] = v
+            obj = {k: v}
 
-    return cfg_params, unk_params
+        nested_update(cfg_params, obj)
+
+    return cfg_params
 
 
-def prepare_hyper_search(cfg_kwargs: dict, cfg_cls: (Configuration, type(Configuration)),
+def prepare_hyper_search(cfg_kwargs: dict,
                          reporthook=None, final_reporthook=None,
                          primary_key=None, max_key=True, reporter_cls=None, with_keys: (list, str, None) = None,
+                         final_keys: (list, str, None) = None,
                          dump=False, disable=False):
     """
+    Updated in v1.3.18
+
     从 nni package 中获取超参，更新配置文件参数。当 nni 不可用或不是 nni 搜索模式时，参数将不会改变。
 
     ..code-block :: python
@@ -199,20 +202,22 @@ def prepare_hyper_search(cfg_kwargs: dict, cfg_cls: (Configuration, type(Configu
     ----------
     cfg_kwargs: dict
         待传入cfg的参数
-    cfg_cls: type(Configuration) or Configuration
-        配置文件
     reporthook
     final_reporthook
     primary_key:
         评估模型用的主键,
         ``nni.report_intermediate_result`` 和 ``nni.report_final_result``中 ``metric`` 的 ``default``
-    max_key
+    max_key: bool
+        主键是越大越好
     reporter_cls
     with_keys: list or str
-        其它要存储的 metric
+        其它要存储的 metric，final report时默认为 primary_key 最优时指标
+    final_keys: list or str
+        with_keys 中使用最后一个 report result 而不是 primary_key 最优时指标
     dump: bool
         为 True 时，会修改 配置文件 中 workspace 参数为 ``workspace/nni.get_experiment_id()/nni.get_trial_id()``
         使得 nni 的中间结果会被存储下来。
+    disable
 
     Returns
     -------
@@ -291,6 +296,7 @@ def prepare_hyper_search(cfg_kwargs: dict, cfg_cls: (Configuration, type(Configu
             def final(self):
                 best_fn = get_min if max_key is False else get_max
                 _with_keys = (with_keys if with_keys else []) + [primary_key]
+                _final_keys = set(final_keys if final_keys else [])
                 final_result = best_fn(
                     self.datas, primary_key, with_keys=";".join(_with_keys), merge=False
                 )
@@ -299,17 +305,18 @@ def prepare_hyper_search(cfg_kwargs: dict, cfg_cls: (Configuration, type(Configu
                 }
                 appendix_dict = dict(final_result[1][primary_key])
                 for key in _with_keys:
-                    feed_dict[key] = appendix_dict[key]
+                    if key in _final_keys:
+                        feed_dict[key] = get_by_key(self.datas[-1], key_parser(key))
+                    else:
+                        feed_dict[key] = appendix_dict[key]
                 report_final_result(feed_dict)
 
         rc = Reporter() if reporter_cls is None else reporter_cls
         reporthook = reporthook if reporthook is not None else rc.intermediate
         final_reporthook = final_reporthook if final_reporthook is not None else rc.final
-        cfg_cls_params, hyper_params = get_params(get_next_parameter(), cfg_cls)
-        using_nni_tag = True if cfg_cls_params or hyper_params else False
-        cfg_kwargs.update(cfg_cls_params)
-        if "hyper_params" in cfg_kwargs:  # pragma: no cover
-            cfg_kwargs["hyper_params"].update(hyper_params)
+        cfg_cls_params = get_params(get_next_parameter())
+        using_nni_tag = True if cfg_cls_params else False
+        nested_update(cfg_kwargs, cfg_cls_params)
         if using_nni_tag is True and dump is True:  # pragma: no cover
             cfg_kwargs["workspace"] = cfg_kwargs.get("workspace", "") + path_append(
                 nni.get_experiment_id(), nni.get_trial_id(), to_str=True
